//////////////////////////////////////////////////////////////////////////
//
//    Program   : log2ps
//
//    File      : cfg.cpp
//
//    Copyright (c) 1998 Ottar Kvindesland
//              This file is a part of the source code of log2ps. It may
//              freely be altered. This copyright notice must however not
//              be removed or changed. The source code may only be
//              distributed further if it is done free of charge, and any
//              changes has been authorized by Ottar Kvindesland
//////////////////////////////////////////////////////////////////////////

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <conio.h>

#include "struct.h"
#include "ps.h"
#include "l2p_glob.h"
#include "l2p_com.h"
#include "man.h"
#include "o_str.h"
#include "o_files.h"
#include "icons.h"
#include "log-gen.h"

FILE *cfgFile;
char errType[MAXLINE] = "\0";
char defCfgFile[15];

/*************************************************************************

	 CHANGING COMMAND AND PARAMETER DEFINITIONS

		       procedure


- STRUCT.H      Introduce new definition
- CFG.CPP       Introduce new definition in the following functions:
		   - cfg_setDef
		   - cfg_show
		   - redefCfg
		   - writeCfgFile
		   - readCfgFile
- MAN.CPP       Update manual

*************************************************************************/

//////////////////////////////////////////////////////////////////////////
//
//        getQPosElement
//
//        Gets a QSO element from string
//////////////////////////////////////////////////////////////////////////

bool getQPosElement (char *s, tlpos *l)
{
char hs[3] = "\0";
char ts[3] = "\0";
int  h;
int  t;
int  sl;
int  i;
bool res = true;

	sl = strlen(s);
	if (sl == 6)
	{	for (i=0; i<sl; i++)
			{s[i] = toupper(s[i]);
			};

		subStr (s, hs, 1, 2);
		subStr (s, ts, 4, 5);

		if (isint(hs) && isint(ts) &&
		    ((s[0] == 'A') || (s[0] == 'G') || (s[0] == 'E')) &&
		    ((s[3] == 'A') || (s[3] == 'G') || (s[3] == 'E')))
		{
			h = atoi(hs);
			t = atoi(ts);
			if ((h * t >= 0) &&
			    (h <= 80)    &&
			    (t <= 80 )   &&
			    (h <= t))
			{	(*l).hd_t = s[0];
				(*l).tl_t = s[3];
				(*l).hd = h;
				(*l).tl = t;
			}
			else
			{	strcpy(errType, "\nQSL pos. betw. 0 and 80 and in sequence");
				res = false;
			};

		}
		else
		{	strcpy(errType, "\nQSL pos. elements format 'CIICII' C:Char, N:Num");
			res = false;
		};
	}
	else
	{	strcpy(errType, "\nQSL position element is 6 chars eg 'A09A11'");
		res = false;
	};

	return(res);

}; /* getQPosElement */


//////////////////////////////////////////////////////////////////////////
//
//      write_qslPos
//
//	Place one set of QSO elements to config parameter
//////////////////////////////////////////////////////////////////////////

void write_qslPos (char *s, tlpos q)
{
char str[3] = "\0";
char *ps;
int  l;

	ps = s;
	ps[0] = q.hd_t;
	ps[1] = '\0';
	formatIntStr (q.hd, 2, str);
	strcat(s, str);
	strcpy (str, "\0");
	l = strlen(s);
	ps[l] = q.tl_t;
	ps[++l] = '\0';
	formatIntStr (q.tl, 2, str);
	strcat(s, str);

};	/* write_qslPos */

//////////////////////////////////////////////////////////////////////////
//
//      write_a_qslPos
//
//	Placing all QSO elements to config file
//////////////////////////////////////////////////////////////////////////

void write_a_qslPos (char *s, int e)
{

	switch (e)
	{	case 1 : write_qslPos (s, c_qp.wrk);  break;
		case 2 : write_qslPos (s, c_qp.yr);  break;
		case 3 : write_qslPos (s, c_qp.mon);  break;
		case 4 : write_qslPos (s, c_qp.day);  break;
		case 5 : write_qslPos (s, c_qp.hr);  break;
		case 6 : write_qslPos (s, c_qp.min);  break;
		case 7 : write_qslPos (s, c_qp.frq);  break;
		case 8 : write_qslPos (s, c_qp.mod);  break;
		case 9 : write_qslPos (s, c_qp.rpt);  break;
	};

};	/* write_a_qslPos */

//////////////////////////////////////////////////////////////////////////
//
//      initCfg
//
//	Setting default config data for all cfg params
//////////////////////////////////////////////////////////////////////////

bool initCfg (cfgElType *c, char *data, int min, int max, char type)
{
bool res = false;
int n;
float r;
char s1[MAXLINE];
char s2[MAXLINE];

	(*c).min = min;
	(*c).max = max;
	(*c).type = type;

	if (max >= min)
	{
	switch (type)
	{case 'c' : if ((strlen(data) <= max) && (strlen(data) >= min))
			res = true;
		    else
		    {	itoa(min, s1, 10);
			itoa(max, s2, 10);
			strcpy(errType, "\nString length: ");
			strcat(errType, data);
			strcpy(errType, ". Must be between ");
			strcat(errType, s1);
			strcpy(errType, " and ");
			strcat(errType, s2);
		    };

		    break;
	 case 'i' : if (isint(data))
		    {	n = atoi(data);
			if ((n <= max) && (n >= min))
			{	res = true;
				itoa(min, s1, 10);
				itoa(max, s2, 10);
				strcpy(errType, "\nValue: ");
				strcat(errType, data);
				strcpy(errType, ". Must be between ");
				strcat(errType, s1);
				strcpy(errType, " and ");
				strcat(errType, s2);
				(*c).i = n;
			};
		    };
		    break;
	 case 'f' : hdtostr(s1, data, '.');
		    tailtostr(s2, data, '.');
		    if (isint(s1) && isint(s2))
		    {	r = atof(data);
			n = atoi(s1);
			if ((n <= max) && (n >= min))
			{	res = true;
				(*c).f = r;
			} else
			{	itoa(min, s1, 10);
				itoa(max, s2, 10);
				strcpy(errType, "\nValue: ");
				strcat(errType, data);
				strcpy(errType, ". Must be between ");
				strcat(errType, s1);
				strcpy(errType, " and ");
				strcat(errType, s2);
			};
		    } else
		    {	    strcpy(errType, "\nString: ");
			    strcat(errType, data);
			    strcpy(errType, ". Must be real with at least 1 decimal");
		    };
		    break;
	 case 'b' : if ((strlen(data) == 1) && is_0or1(data) )
		    {	res = true;
			(*c).b = (strcmp(data, "1") == 0);
		    };
		    break;
	};
	};

	if (res)
		strcpy((*c).s, data);

	return (res);

};	/* initCfg */

//////////////////////////////////////////////////////////////////////////
//
//      chooseCfg
//
//	Initiating a config parameter, and sets boolean flag
//	if data is maintained.
//	Used when parameter not used equals '0'
//////////////////////////////////////////////////////////////////////////

bool chooseCfg (cfgElType *c, char *data, int min, int max, char type)
{
bool res=false;

	if(initCfg (c, data, min, max, type))
	{	(*c).b = (strcmp((*c).s, "0") != 0);
		res = true;
	};

	return (res);

};	/* chooseCfg */

//////////////////////////////////////////////////////////////////////////
//
//      loadCfg
//
//	Loading char value to config
//////////////////////////////////////////////////////////////////////////

bool loadCfg (cfgElType *c, char *data)
{
int min, max;
char type;

	max = (*c).max;
	min = (*c).min;
	type = (*c).type;

	return (initCfg (c, data, min, max, type));

};	/* loadCfg */

//////////////////////////////////////////////////////////////////////////
//
//      selectCfg
//
//	Loading char value to config. Sets params via chooseCfg
//////////////////////////////////////////////////////////////////////////

bool selectCfg (cfgElType *c, char *data)
{
int min, max;
char type;

	max = (*c).max;
	min = (*c).min;
	type = (*c).type;

	return (chooseCfg (c, data, min, max, type));

};	/* selectCfg */

//////////////////////////////////////////////////////////////////////////
//
//        el_setDefs
//
//////////////////////////////////////////////////////////////////////////

void el_setDefs (void)
{
	getQPosElement("A15A25", &c_qp.wrk);
	getQPosElement("A06A07", &c_qp.yr);
	getQPosElement("A02A04", &c_qp.mon);
	getQPosElement("A00A01", &c_qp.day);
	getQPosElement("A09A10", &c_qp.hr);
	getQPosElement("A12A13", &c_qp.min);
	getQPosElement("A26A32", &c_qp.frq);
	getQPosElement("A33A37", &c_qp.mod);
	getQPosElement("A38A41", &c_qp.rpt);

};	/* el_setDefs */

//////////////////////////////////////////////////////////////////////////
//
//        ct_setDefs
//
//////////////////////////////////////////////////////////////////////////

void ct_setDefs (void)
{
	getQPosElement("A35A46", &c_qp.wrk);
	getQPosElement("A11A12", &c_qp.yr);
	getQPosElement("A07A09", &c_qp.mon);
	getQPosElement("A04A05", &c_qp.day);
	getQPosElement("A15A16", &c_qp.hr);
	getQPosElement("A17A18", &c_qp.min);
	getQPosElement("A28A34", &c_qp.frq);
	getQPosElement("A42A48", &c_qp.mod);
	getQPosElement("A47A51", &c_qp.rpt);

	selectCfg(&cfg.fixmod, "SSB");
	selectCfg(&cfg.bandFrq, "BAND");

};	/* ct_setDefs */

//////////////////////////////////////////////////////////////////////////
//
//        k1ea_setDefs
//
//////////////////////////////////////////////////////////////////////////

void k1ea_setDefs (void)
{
	getQPosElement("A38A49", &c_qp.wrk);
	getQPosElement("A10A11", &c_qp.yr);
	getQPosElement("A06A08", &c_qp.mon);
	getQPosElement("A03A04", &c_qp.day);
	getQPosElement("A14A15", &c_qp.hr);
	getQPosElement("A16A17", &c_qp.min);
	getQPosElement("A33A36", &c_qp.frq);
	getQPosElement("A20A25", &c_qp.mod);
	getQPosElement("A51A53", &c_qp.rpt);

	selectCfg(&cfg.bandFrq, "BAND");

};	/* k1ea_setDefs */

//////////////////////////////////////////////////////////////////////////
//
//      cfg_setDef
//
//	Setting default config data
//////////////////////////////////////////////////////////////////////////

void cfg_setDef (void)
{
bool res = true;

	el_setDefs ();

	res = (res && initCfg(&cfg.logFileName, "K1EA.LOG", 0, 80, 'c'));
	res = (res && initCfg(&cfg.mySig, "LA9IHA", 2, 15, 'c'));
	res = (res && initCfg(&cfg.myName, "Ottar Kvindesland", 0, 80, 'c'));
	res = (res && initCfg(&cfg.myQTH,
		"WALA-C,WANCA-C17,WANLO-JO59HQ,Zone:ITU-18,CQ-14", 0, 80, 'c'));
	res = (res && chooseCfg(&cfg.myRig, "Howes TX2000, DC2000", 0, 80, 'c'));
	res = (res && chooseCfg(&cfg.pQTH, "Portable from the garden", 0, 80, 'c'));
	res = (res && chooseCfg(&cfg.banner, "SAMPLE",   0, 25, 'c'));
	res = (res && chooseCfg(&cfg.myMail, log2ps_url, 0, 60, 'c'));
	res = (res && chooseCfg(&cfg.myBBS, "0",         0, 80, 'c'));
	res = (res && chooseCfg(&cfg.logo, "NRRL",       0, 7, 'c'));
	res = (res && initCfg(&cfg.lshade, "0.86",     0, 3, 'f'));
	res = (res && initCfg(&cfg.cardsPerSheet, "2", 1, 4, 'i'));
	res = (res && initCfg(&cfg.x_offset, "0",      -999, 999, 'i'));
	res = (res && initCfg(&cfg.myAddress,
		"Belsjoevn. 5 1440 Droebak, Norway", 0, 80, 'c'));
	res = (res && initCfg(&cfg.logProg, "K1EA", 0, 8, 'c'));
	res = (res && initCfg(&cfg.century, "19",      19, 99, 'i'));
	res = (res && initCfg(&cfg.corners, "1",       0, 0, 'b'));
	res = (res && initCfg(&cfg.frame, "0",         0, 0, 'b'));
	res = (res && initCfg(&cfg.dest, "LOG.PS",     0, 80, 'c'));
	res = (res && initCfg(&cfg.last_yr, "0",       0,  5, 'c'));
	res = (res && initCfg(&cfg.last_mon, "0",      0, 12, 'c'));
	res = (res && initCfg(&cfg.last_day, "0",      0, 31, 'i'));
	res = (res && initCfg(&cfg.last_time, "0",     0, 10, 'c'));
	res = (res && initCfg(&cfg.dupl, "0",          0, 0, 'b'));
	res = (res && initCfg(&cfg.bandFrq, "MHz",     0, 10, 'c'));
	res = (res && initCfg(&cfg.greeting,  "PSE QSL, 73", 0, 40, 'c'));
	res = (res && initCfg(&cfg.expert, "0",        0, 0, 'b'));
	res = (res && initCfg(&cfg.y_offset, "0",      -999, 999, 'i'));
	res = (res && initCfg(&cfg.title,
		"I have the pleasure of confirming our QSO:", 0, 60, 'c'));
	res = (res && initCfg(&cfg.xchar,  "0",        0, 0, 'b'));
	res = (res && initCfg(&cfg.edge, "0",          0, 0, 'b'));
	res = (res && initCfg(&cfg.country, "NORWAY",  0, 80, 'c'));
	res = (res && initCfg(&cfg.modlbl, "MODE",     0, 10, 'c'));
	res = (res && chooseCfg(&cfg.fixmod, "0",      0, 10, 'c'));
	res = (res && initCfg(&cfg.ptitle, "1",        0,  0, 'b'));
	res = (res && initCfg(&cfg.fixrpt, "0",        0, 10, 'c'));
	res = (res && initCfg(&cfg.margin, "18",   -400, 400, 'i'));
	res = (res && initCfg(&cfg.icons, "0",          0, 0, 'b'));
	res = (res && initCfg(&cfg.oscall, "0",        0, 80, 'c'));

	if (!res)
	{	printf("\n\nCONFIGURATION PARAMETER ERROR -");
		printf(" SEE LAST DEFINED BY -SHCFG\n");
		printf("PLEASE SEND ERROR INFO TO LA9IHA\n");
		printf("%s\n", errType);
	};


};	/* cfg_setDef */


//////////////////////////////////////////////////////////////////////////
//
//      setQslPos
//
//      Setting QSO position elements in log output
//////////////////////////////////////////////////////////////////////////

bool setQslPos (char *lp)
{
bool r = false;

	if (strcmp(lp, "LOG-EQF") == 0)
	{	el_setDefs();
		r = true;
	};
	if (strcmp(lp, "CT") == 0)
	{	ct_setDefs();
		r = true;
	};
	if (strcmp(lp, "K1EA") == 0)
	{	k1ea_setDefs();
		r = true;
	};
	if (strcmp(lp, "OTHER") == 0)
		r = true;

	return (r);

};	/* setQslPos */


//////////////////////////////////////////////////////////////////////////
//
//        chk......
//
//	Various functions under common title header which performs checks
//	on parameters to program
//
//////////////////////////////////////////////////////////////////////////

bool chk_x_offset(cfgElType *c, char *p)
{
bool  r      = true;
bool  relpos = false;
char reschar[MAXLINE];
int  resint, crds;

char *s;

	s = strupr(p);

	crds = cfg.cardsPerSheet.i;
	strcpy(reschar, s);

	if (strcmp(s, "L") == 0)
	  {switch (crds)
		{case 1 : resint = -157; break;
		 case 2 : resint = -157; break;
		 case 3 : resint = -104; break;
		 case 4 : resint = 42;   break;
		 default: r = false; 	 break;
		};
	   relpos = true;
	  };
	if (strcmp(s, "R") == 0)
	  {switch (crds)
		{case 1 : resint = 168; break;
		 case 2 : resint = 168; break;
		 case 3 : resint = 105; break;
		 case 4 : resint = -45; break;
		 default: r = false;    break;
		};
	   relpos = true;
	  };
	if (strcmp(s, "C") == 0)
	{	relpos = true;
		resint = 0;
	};

	if (relpos)
	{       strcpy(reschar, "\0"); // Be sure it's reset, C  remember...
		itoa(resint, reschar, 10);
	};

	if (r)
		r = loadCfg(c, reschar);
	else
		strcpy(errType, "Cannot define relative X, unknown parameter");
	return(r);

};  /* chk_x_offset */

bool chk_logProg (cfgElType *s, char *p)
{
bool r;
char *ps;

	ps = strupr(p);
	r = setQslPos(ps);

	if (r)
	     loadCfg(s, ps);
	else
	     strcpy(errType, "Illegal log prog value, legals found with switch");
	return (r);

};    /* */
bool chk_blanks (char *s)
{bool r;
	r = isint(s);
	if (r)
		r = (atoi(s) > 0);

	if (r)
		QSL_blanks(atoi(s));
	else
	     strcpy(errType, "Illegal value, expected integer greater than 0");
	return (r);
};  /* chk_blanks */

bool chk_setDef (void)
{char c;
	printf("Do you wish to replace current configuration with default (Y/N)");
	c = getc(stdin);
	if ((c == 'Y') || (c == 'y'))
	{
		cfg_setDef ();
		printf("\nDone");
	};
	return(true);
};  /* chk_SetDef */

//////////////////////////////////////////////////////////////////////////
//
//        shQPosElement
//
//        Prints QSO element positions from struct
//////////////////////////////////////////////////////////////////////////

void shQPosElement (char *s, tlpos l)
{
char t[MAXLINE] = "\0";

	strcpy(s, "                 ");
	formatIntStr (l.hd, 2, t);
	strcat(s, t);
	s[12] = l.hd_t;
	t[0] = '\0';
	strcat(s, "               ");
	formatIntStr (l.tl, 2, t);
	strcat(s, t);
	s[29] = l.tl_t;

};	/* shQPosElement */

//////////////////////////////////////////////////////////////////////////
//
//        shQPos
//
//        Prints QSO element positions from log
//////////////////////////////////////////////////////////////////////////

void shQPos (char *s, int n)
{
char t[MAXLINE];

	switch (n)
	{	case 1 : strcpy (s, "\n        QSO-                Head       ");
			 strcat(s, "      Tail\0");
			 break;
		case 2 : strcpy (s, "        Element          Type  Posn    ");
			 strcat(s, "   Type  Posn\0");
			 break;
		case 3 : strcpy (s, "        ------------------------------");
			 strcat(s, "--------------\0");
			 break;
		case 4 : strcpy (s, "        Stn wrkd");
			 strcpy(t, "\0");
			 shQPosElement (t, c_qp.wrk);
			 strcat(s, t);
			 break;
		case 5 : strcpy (s, "        Year    ");
			 strcpy(t, "\0");
			 shQPosElement (t, c_qp.yr);
			 strcat(s, t);
			 break;
		case 6 : strcpy (s, "        Month   ");
			 strcpy(t, "\0");
			 shQPosElement (t, c_qp.mon);
			 strcat(s, t);
			 break;
		case 7 : strcpy (s, "        Day     ");
			 strcpy(t, "\0");
			 shQPosElement (t, c_qp.day);
			 strcat(s, t);
			 break;
		case 8 : strcpy (s, "        Hour    ");
			 strcpy(t, "\0");
			 shQPosElement (t, c_qp.hr);
			 strcat(s, t);
			 break;
		case 9 : strcpy (s, "        Minutes ");
			 strcpy(t, "\0");
			 shQPosElement (t, c_qp.min);
			 strcat(s, t);
			 break;
		case 10: strcpy (s, "        Frq/band");
			 strcpy(t, "\0");
			 shQPosElement (t, c_qp.frq);
			 strcat(s, t);
			 break;
		case 11: strcpy (s, "        Mode    ");
			 strcpy(t, "\0");
			 shQPosElement (t, c_qp.mod);
			 strcat(s, t);
			 break;
		case 12: strcpy (s, "        Report  ");
			 strcpy(t, "\0");
			 shQPosElement (t, c_qp.rpt);
			 strcat(s, t);
			 break;
	};
};	/* shQPos */

//////////////////////////////////////////////////////////////////////////
//
//      cfg_present
//
//	Display one set of config data
//////////////////////////////////////////////////////////////////////////

void cfg_present (char *in, cfgElType c, char *out)
{
int n = 0;

	strcpy(out, in);

	for (n = strlen(in); n < 21; n++)
	{
		strcat(out, " ");
	};
	if ((c.type == 'b') ||
	   ((c.type == 'c') &&
	    (strlen(c.s) == 1) &&
	    (strcmp(c.s, "0") == 0)) )
	{	if (c.b) strcat(out, "YES");
		else strcat(out, "NO");
	} else
	strcat(out, c.s);

};	/* cfg_present */

//////////////////////////////////////////////////////////////////////////
//
//      cfg_show
//
//	Display config data at -shcfg
//////////////////////////////////////////////////////////////////////////

void cfg_show (void)
{
int n = 0;
int lineNo = 0;
bool ferdig = false;
char str[MAXLINE];
char ch = ' ';

	printf("\n");

	while (!ferdig)
	{
		switch (n++)
		{
		case  0 : cfg_present ("Log file", cfg.logFileName, str); break;
		case  1 : cfg_present ("My signature", cfg.mySig, str); break;
		case  2 : cfg_present ("My name", cfg.myName, str); break;
		case  3 : cfg_present ("My QTH locator", cfg.myQTH, str); break;
		case  4 : cfg_present ("My rig", cfg.myRig, str); break;
		case  5 : cfg_present ("Portable QTH", cfg.pQTH, str); break;
		case  6 : cfg_present ("Banner", cfg.banner, str); break;
		case  7 : cfg_present ("Internet address", cfg.myMail, str); break;
		case  8 : cfg_present ("AX.25 packet address", cfg.myBBS, str); break;
		case  9 : cfg_present ("Logo", cfg.logo, str); break;
		case 10 : cfg_present ("Logo shade", cfg.lshade, str); break;
		case 11 : cfg_present ("Cards per sheet", cfg.cardsPerSheet, str); break;
		case 12 : cfg_present ("X offset", cfg.x_offset, str); break;
		case 13 : cfg_present ("My postal address", cfg.myAddress, str); break;
		case 14 : cfg_present ("Log program used", cfg.logProg, str); break;
		case 15 : cfg_present ("Century", cfg.century, str); break;
		case 16 : cfg_present ("Corners", cfg.corners, str); break;
		case 17 : cfg_present ("Frame", cfg.frame, str); break;
		case 18 : cfg_present ("Destination", cfg.dest, str); break;
		case 19 : cfg_present ("Frequency box in", cfg.bandFrq, str); break;
		case 20 : cfg_present ("Greeting", cfg.greeting, str); break;
		case 21 : cfg_present ("Expert status", cfg.expert, str); break;
		case 22 : cfg_present ("Y offset", cfg.y_offset, str); break;
		case 23 : cfg_present ("Title", cfg.title, str); break;
		case 24 : cfg_present ("Extended characters", cfg.xchar, str); break;
		case 25 : cfg_present ("Card edges", cfg.edge, str); break;
		case 26 : cfg_present ("Country", cfg.country, str); break;
		case 27 : cfg_present ("Mode label", cfg.modlbl, str); break;
		case 28 : cfg_present ("Fixed mode", cfg.fixmod, str); break;
		case 29 : cfg_present ("Opr's titles", cfg.ptitle, str); break;
		case 30 : cfg_present ("Fixed rpt", cfg.fixrpt, str); break;
		case 31 : cfg_present ("Left margin", cfg.margin, str); break;
		case 32 : cfg_present ("Icons", cfg.icons, str); break;
		case 33 : cfg_present ("Operating Sys. Call", cfg.oscall, str); break;

		case  34 : strcpy(str, " ");
		n = 100;   break;
		case 100 :shQPos(str, 1);          break;
		case 101 :shQPos(str, 2);          break;
		case 102 :shQPos(str, 3);          break;
		case 103 :shQPos(str, 4);          break;
		case 104 :shQPos(str, 5);          break;
		case 105 :shQPos(str, 6);          break;
		case 106 :shQPos(str, 7);          break;
		case 107 :shQPos(str, 8);          break;
		case 108 :shQPos(str, 9);          break;
		case 109 :shQPos(str, 10);         break;
		case 110 :shQPos(str, 11);         break;
		case 111 :shQPos(str, 12);         break;

		case 112 :ferdig = true;
		strcpy(str, "  ");
		};

		if (ch != 'Q')
		{
			puts(str);
			if (lineNo++ > 19)
			{
				printf("\n..........Press Q to quit, any key to continue\n");
				ch = toupper(getche());
				lineNo = 0;
			};
		};
	};

}; /* cfg_show */

//////////////////////////////////////////////////////////////////////////
//
//      redefCfg
//
//	Handling switches at command line
//////////////////////////////////////////////////////////////////////////

bool redefCfg (char *s)
{
int n = 0;
bool res = true;
bool unknown = false;
bool done = false;

/* Must use pointers. Arrays causes parameter to be deleted in functions
   hdtostr and tailtostr.
*/

char *c = (char *) malloc ((sizeof (char) * MAXLINE));
char *p = (char *) malloc ((sizeof (char) * MAXLINE));

	strcpy(errType, "\0");

	if (strlen(s) > MAXLINE)
	{
		printf("%s Max length of switch is 80 characters\n", errType);
		return(false);
	};

	hdtostr(c, s, '=');
	tailtostr(p, s, '=');

	while (!done)
	{switch (n++)
		{
		case 0  : if (stricmp(c, "-?") == 0)
			  {	man_out();
			  done = true;}; break;
		case 1  : if (stricmp(c, "?") == 0)
			  { 	man_out();
			  done = true;}; break;
		case 2  : if (stricmp(c, "-BANDFRQ") == 0)
			  {	res = (loadCfg (&cfg.bandFrq, p));
			  done = true;}; break;
		case 3  : if (stricmp(c, "-BANNER") == 0)
			  {	res = selectCfg(&cfg.banner, replus(p));
			  done = true;}; break;
		case 4  : if (stricmp(c, "-BLANKS") == 0)
			  {	res = chk_blanks(p);
			  done = true;}; break;
		case 5  : if (stricmp(c, "-CENTURY") == 0)
			  {	res = loadCfg(&cfg.century, p);
			  done = true;}; break;
		case 6  : if (stricmp(c, "-CORNERS") == 0)
			  {	res = loadCfg(&cfg.corners, p);
			  done = true;}; break;
		case 7  : if (stricmp(c, "-COUNTRY") == 0)
			  {	res = loadCfg (&cfg.country, replus(p));
			  done = true;}; break;
		case 8  : if (stricmp(c, "-DEST") == 0)
			  {	res = loadCfg(&cfg.dest, p);
			  done = true;}; break;
		case 9  : if (stricmp(c, "-EDGE") == 0)
			  {	res = loadCfg(&cfg.edge, p);
			  done = true;}; break;
		case 10 : if (stricmp(c, "-FIXMOD") == 0)
			  {	res = selectCfg(&cfg.fixmod, p);
			  done = true;}; break;
		case 11 : if (stricmp(c, "-FIXRPT") == 0)
			  {	res = (selectCfg (&cfg.fixrpt, p));
			  done = true;}; break;
		case 12 : if (stricmp(c, "-FRAME") == 0)
			  {	res = loadCfg(&cfg.frame, p);
			  done = true;}; break;
		case 13 : if (stricmp(c, "-GREETING") == 0)
			  {	res = (loadCfg (&cfg.greeting, replus(p)));
			  done = true;}; break;
		case 14 : if (stricmp(c, "H") == 0)
			  {	man_out();
			  done = true;}; break;
		case 15 : if (stricmp(c, "HELP") == 0)
			  {	man_out();
			  done = true;}; break;
		case 16 : if (stricmp(c, "-H") == 0)
			  {	man_out();
			  done = true;}; break;
		case 17 : if (stricmp(c, "-HELP") == 0)
			  {	man_out();
			  done = true;}; break;
		case 18 : if (stricmp(c, "-ICONS") == 0)
			  {	res = (loadCfg (&cfg.icons, p));
				if (cfg.icons.b)
					setIcons();
			  done = true;}; break;
		case 19 : if (stricmp(c, "-LOGFILE") == 0)
			  {	res = loadCfg(&cfg.logFileName, p);
			  done = true;}; break;
		case 20 : if (stricmp(c, "-LOGO") == 0)
			  {	res = (selectCfg (&cfg.logo, p));
			  done = true;}; break;
		case 21 : if (stricmp(c, "-LSHADE") == 0)
			  {	res = selectCfg(&cfg.lshade, p);
			  done = true;}; break;
		case 22 : if (stricmp(c, "MAN") == 0)
			  {	man_out();
			  done = true;}; break;
		case 23 : if (stricmp(c, "-MAN") == 0)
			  {	man_out();
			  done = true;}; break;
		case 24 : if (stricmp(c, "-MARGIN") == 0)
			  {	res = (loadCfg (&cfg.margin, p));
			  done = true;}; break;
		case 25 : if (stricmp(c, "-MODLBL") == 0)
			  {	res = (loadCfg (&cfg.modlbl, replus(p)));
			  done = true;}; break;
		case 26 : if (stricmp(c, "-myAddr") == 0)
			  {	res = (loadCfg (&cfg.myAddress, replus(p)));
			  done = true;}; break;
		case 27 : if (stricmp(c, "-MYBBS") == 0)
			  {	res = (selectCfg (&cfg.myBBS, p));
			  done = true;}; break;
		case 28 : if (stricmp(c, "-MYLOC") == 0)
			  {	res = (loadCfg (&cfg.myQTH, replus(p)));
			  done = true;}; break;
		case 29 : if (stricmp(c, "-MYMAIL") == 0)
			  {	res = (selectCfg (&cfg.myMail, p));
			  done = true;}; break;
		case 30 : if (stricmp(c, "-MYNAME") == 0)
			  {	res = (loadCfg (&cfg.myName, replus(p)));
			  done = true;}; break;
		case 31 : if (stricmp(c, "-MYRIG") == 0)
			  {	res = (selectCfg (&cfg.myRig, replus(p)));
			  done = true;}; break;
		case 32:  if (stricmp(c, "-MYSIG") == 0)
			   {	res = (loadCfg (&cfg.mySig, p));
			   done = true;}; break;
		case 33 : if (stricmp(c, "-OSCALL") == 0)
			  {	res = (selectCfg (&cfg.oscall, replus(p)));
			  done = true;}; break;
		case 34 : if (stricmp(c, "-PTITLE") == 0)
			  {	res = loadCfg(&cfg.ptitle, p);
			  done = true;}; break;
		case 35 : if (stricmp(c, "-PQTH") == 0)
			  {	res = (selectCfg (&cfg.pQTH, replus(p)));
			  done = true;}; break;
		case 36 : if (stricmp(c, "-QPDAY") == 0)
			  {	res = getQPosElement (p, &c_qp.day);
			  done = true;}; break;
		case 37 : if (stricmp(c, "-QPFRQ") == 0)
			  {	res = getQPosElement (p, &c_qp.frq);
			  done = true;}; break;
		case 38 : if (stricmp(c, "-QPHR") == 0)
			  {	res = getQPosElement (p, &c_qp.hr);
			  done = true;}; break;
		case 39 : if (stricmp(c, "-QPMIN") == 0)
			  {	res = getQPosElement (p, &c_qp.min);
			  done = true;}; break;
		case 40 : if (stricmp(c, "-QPMOD") == 0)
			  {	res = getQPosElement (p, &c_qp.mod);
			  done = true;}; break;
		case 41 : if (stricmp(c, "-QPMON") == 0)
			  {	res = getQPosElement (p, &c_qp.mon);
			  done = true;}; break;
		case 42 : if (stricmp(c, "-QPRPT") == 0)
			  {	res = getQPosElement (p, &c_qp.rpt);
			  done = true;}; break;
		case 43 : if (stricmp(c, "-QPTEST") == 0)
			  {	qp_testlog();
			  done = true;}; break;
		case 44 : if (stricmp(c, "-QPWRK") == 0)
			  {	res = getQPosElement (p, &c_qp.wrk);
			  done = true;}; break;
		case 45 : if (stricmp(c, "-QPYR") == 0)
			  {	res = getQPosElement (p, &c_qp.yr);
			  done = true;}; break;
		case 46 : if (stricmp(c, "-QSLS") == 0)
			  {	res = loadCfg(&cfg.cardsPerSheet, p);
			  done = true;}; break;
		case 47 : if (stricmp(c, "-SETDEF") == 0)
			  {	res = chk_setDef();
			  done = true;}; break;
		case 48 : if (stricmp(c, "-SHCFG") == 0)
			  {	cfg_show();
			  done = true;}; break;
		case 49 : if (stricmp(c, "-SLOG") == 0)
			  {	res = chk_logProg(&cfg.logProg, p);
			  done = true;}; break;
		case 50 : if (stricmp(c, "-TITLE") == 0)
			  {	res = (loadCfg (&cfg.title, replus(p)));
			  done = true;}; break;
		case 51 : if (stricmp(c, "-XCHAR") == 0)
			  {	res = loadCfg(&cfg.xchar, p);
			  done = true;}; break;
		case 52 : if (stricmp(c, "-X") == 0)
			  {	res = loadCfg(&cfg.expert, p);
			  done = true;}; break;
		case 53 : if (stricmp(c, "-XOFFSET") == 0)
			  {	res = chk_x_offset(&cfg.x_offset, p);
			  done = true;}; break;
		case 54 : if (stricmp(c, "-YOFFSET") == 0)
			  {	res = loadCfg(&cfg.y_offset, p);
			  done = true;}; break;

		case 55 : if (stricmp(c, "-DUPL") == 0)
			  {	res = loadCfg(&cfg.dupl, p);
			  done = true;}; break;


		case 56 : unknown = true;
			  done = true;
			  res = true; break;
		};
		if (!res)
		{	res = true;
			printf("\nSwitch error: %s. Illegal parameter: %s.\n", c, p);
			printf("%s\n", errType);
		};
		if (unknown)
		{	unknown = false;
			res = false;
			printf("Unknown command: %s ", c);
		};
	};

	free(c);
	free(p);

	return (res);

};	/* redefCfg */

//////////////////////////////////////////////////////////////////////////
//
//      writeCfgFile
//
//	Places current config data to config file
//////////////////////////////////////////////////////////////////////////

bool writeCfgFile (void)
{
FILE *pFCfg;
bool finished = false;
bool res = true;
int n = 0;
char cfgStr[MAXLINE] = "\0";

	pFCfg = fopen(defCfgFile, "wt");

	if ((cfg.myMail.b) && (cfg.myBBS.b))
		finished = !selectCfg(&cfg.myMail, "0");

	while (!finished)
	{
		switch (n++)
		{
			case 0  : strcpy(cfgStr, cfg.logFileName.s);    break;
			case 1  : strcpy(cfgStr, cfg.mySig.s);          break;
			case 2  : strcpy(cfgStr, cfg.myName.s);         break;
			case 3  : strcpy(cfgStr, cfg.myQTH.s);          break;
			case 4  : strcpy(cfgStr, cfg.myRig.s);          break;
			case 5  : strcpy(cfgStr, cfg.pQTH.s);           break;
			case 6  : strcpy(cfgStr, cfg.banner.s);         break;
			case 7  : strcpy(cfgStr, cfg.myMail.s);         break;
			case 8  : strcpy(cfgStr, cfg.myBBS.s);          break;
			case 9  : strcpy(cfgStr, cfg.logo.s);           break;
			case 10 : strcpy(cfgStr, cfg.lshade.s);         break;
			case 11 : strcpy(cfgStr, cfg.cardsPerSheet.s);  break;
			case 12 : strcpy(cfgStr, cfg.x_offset.s);       break;
			case 13 : strcpy(cfgStr, cfg.logProg.s);        break;
			case 14 : strcpy(cfgStr, cfg.myAddress.s);      break;
			case 15 : strcpy(cfgStr, cfg.century.s);        break;
			case 16 : strcpy(cfgStr, cfg.corners.s);        break;
			case 17 : strcpy(cfgStr, cfg.frame.s);          break;
			case 18 : strcpy(cfgStr, cfg.dest.s);           break;
			case 19 : strcpy(cfgStr, cfg.last_yr.s);        break;
			case 20 : strcpy(cfgStr, cfg.last_mon.s);       break;
			case 21 : strcpy(cfgStr, cfg.last_day.s);       break;
			case 22 : strcpy(cfgStr, cfg.last_time.s);      break;
			case 23 : strcpy(cfgStr, cfg.dupl.s);           break;
			case 24 : strcpy(cfgStr, cfg.bandFrq.s);        break;
			case 25 : strcpy(cfgStr, cfg.greeting.s);       break;
			case 26 : strcpy(cfgStr, cfg.expert.s);	        break;
			case 27 : strcpy(cfgStr, cfg.y_offset.s);       break;
			case 28 : strcpy(cfgStr, cfg.title.s);	        break;
			case 29 : strcpy(cfgStr, cfg.xchar.s);	        break;
			case 30 : strcpy(cfgStr, cfg.edge.s);	        break;
			case 31 : strcpy(cfgStr, cfg.country.s);        break;
			case 32 : strcpy(cfgStr, cfg.modlbl.s);	        break;
			case 33 : write_a_qslPos (cfgStr, 1);           break;
			case 34 : write_a_qslPos (cfgStr, 2);           break;
			case 35 : write_a_qslPos (cfgStr, 3);           break;
			case 36 : write_a_qslPos (cfgStr, 4);           break;
			case 37 : write_a_qslPos (cfgStr, 5);           break;
			case 38 : write_a_qslPos (cfgStr, 6);           break;
			case 39 : write_a_qslPos (cfgStr, 7);           break;
			case 40 : write_a_qslPos (cfgStr, 8);           break;
			case 41 : write_a_qslPos (cfgStr, 9);           break;
			case 42 : strcpy(cfgStr, cfg.fixmod.s);	        break;
			case 43 : strcpy(cfgStr, cfg.ptitle.s);	        break;
			case 44 : strcpy(cfgStr, cfg.fixrpt.s);	        break;
			case 45 : strcpy(cfgStr, cfg.margin.s);	        break;
			case 46 : strcpy(cfgStr, cfg.icons.s);	        break;
			case 47 : strcpy(cfgStr, cfg.oscall.s);	        break;

			case 48 : finished = true;                     break;
		};
		if (!finished)
		{
			fprintf(pFCfg, "%s", cfgStr);
			fprintf(pFCfg, "\n\0");
		};

	};

	res = (n > 43);
	fclose(pFCfg);

	return (res);

};	// vriteCfgFile

//////////////////////////////////////////////////////////////////////////
//
//      readCfgFile
//
//	Reads config file
//////////////////////////////////////////////////////////////////////////
bool readCfgFile (void)
{
int lNo;
bool noCfgErr = true;
bool f = false;

char l[MAXLINE];

	strcpy(errType, "Config file error: ");

	lNo = 0;

	fseek(cfgFile, 0, SEEK_SET);

	strcpy(errType, "\0");
	while (!fileEnd)
	{
		readLine(l, cfgFile);

		if (!fileEnd)
		switch(lNo++)
		{
		case 0  : f = (loadCfg (&cfg.logFileName, l)); break;
		case 1  : f = (loadCfg (&cfg.mySig, l));  break;
		case 2  : f = (loadCfg (&cfg.myName, l));  break;
		case 3  : f = (loadCfg (&cfg.myQTH, l));  break;
		case 4  : f = (selectCfg (&cfg.myRig, l));  break;
		case 5  : f = (selectCfg (&cfg.pQTH, l));  break;
		case 6  : f = (selectCfg (&cfg.banner, l));  break;
		case 7  : f = (selectCfg (&cfg.myMail, l));  break;
		case 8  : f = (selectCfg (&cfg.myBBS, l));  break;
		case 9  : f = (selectCfg (&cfg.logo, l));  break;
		case 10 : f = (selectCfg (&cfg.lshade, l));  break;
		case 11 : f = (loadCfg (&cfg.cardsPerSheet, l));  break;
		case 12 : f = (loadCfg (&cfg.x_offset, l));  break;
		case 13 : f = (loadCfg (&cfg.logProg, l));  break;
		case 14 : f = (loadCfg (&cfg.myAddress, l));  break;
		case 15 : f = (loadCfg (&cfg.century, l));  break;
		case 16 : f = (loadCfg (&cfg.corners, l));  break;
		case 17 : f = (loadCfg (&cfg.frame, l));  break;
		case 18 : f = (loadCfg (&cfg.dest, l));  break;
		case 19 : f = (loadCfg (&cfg.last_yr, l));  break;
		case 20 : f = (loadCfg (&cfg.last_mon, l));  break;
		case 21 : f = (loadCfg (&cfg.last_day, l));  break;
		case 22 : f = (loadCfg (&cfg.last_time, l));  break;
		case 23 : f = (loadCfg (&cfg.dupl, l));  break;
		case 24 : f = (loadCfg (&cfg.bandFrq, l)); break;
		case 25 : f = (loadCfg (&cfg.greeting, l)); break;
		case 26 : f = (loadCfg (&cfg.expert, l)); break;
		case 27 : f = (loadCfg (&cfg.y_offset, l)); break;
		case 28 : f = (loadCfg (&cfg.title, l)); break;
		case 29 : f = (loadCfg (&cfg.xchar, l)); break;
		case 30 : f = (loadCfg (&cfg.edge, l)); break;
		case 31 : f = (loadCfg (&cfg.country, l)); break;
		case 32 : f = (loadCfg (&cfg.modlbl, l)); break;

		case 33 : getQPosElement (l, &c_qp.wrk); break;
		case 34 : getQPosElement (l, &c_qp.yr); break;
		case 35 : getQPosElement (l, &c_qp.mon); break;
		case 36 : getQPosElement (l, &c_qp.day); break;
		case 37 : getQPosElement (l, &c_qp.hr); break;
		case 38 : getQPosElement (l, &c_qp.min); break;
		case 39 : getQPosElement (l, &c_qp.frq); break;
		case 40 : getQPosElement (l, &c_qp.mod); break;
		case 41 : getQPosElement (l, &c_qp.rpt); break;
		case 42 : f = (selectCfg (&cfg.fixmod, l)); break;
		case 43 : f = (loadCfg (&cfg.ptitle, l)); break;
		case 44 : f = (selectCfg (&cfg.fixrpt, l)); break;
		case 45 : f = (loadCfg (&cfg.margin, l)); break;
		case 46 : f = (loadCfg (&cfg.icons, l)); break;
		case 47 : f = (selectCfg (&cfg.oscall, l)); break;

		case 48 : fileEnd = true;

		noCfgErr = noCfgErr && f;
		};

		if (!f)
		{	printf("\nIllegal config file parameter %s, No: %d\n", l, lNo-1);
			printf(" --> %s\n", errType);
			strcpy(errType, "\0");
		};
	};

	fclose(cfgFile);

	return(noCfgErr);

};	//readCfgFile

//////////////////////////////////////////////////////////////////////////
//
//      confFile
//
//	Open config file to read data
//////////////////////////////////////////////////////////////////////////

byte confFile (void)
{
byte OK = true;

	/* Read configuration file */
	fileEnd    = false;

	if (file_exist(defCfgFile))
	{	cfgFile = fopen (defCfgFile, "rt");
		OK = readCfgFile();
	}
	else
	{	printf("\nNo configuration file detected, creating: %s ", defCfgFile);
		printf("\nHint: Use command log2ps -? to see how to set your own");
		printf("\n      configuration. Check it with command log2ps -shcfg");
		printf("\n");
	};
	return(OK);

};	/* confFile */

//////////////////////////////////////////////////////////////////////////
//
//      getCfg
//
//	Get config info from command line
//////////////////////////////////////////////////////////////////////////

byte getCfg(int argc, char *argv[])
{
int n = 1;
bool OK = true;

	fileEnd    = false;
	cfg_setDef ();
	strcpy(defCfgFile, "log2ps.cfg");
	OK = confFile();

	/* Read command line parameters */
	while ((n < argc) && (OK))
	{
		OK = redefCfg(argv[n]);
		if (OK)
			n++;
		else
			printf ("Illegal switch: %s\n", argv[n]);
	};

	return ((argc == 1) && (OK));

};	/*getCfg */
