//////////////////////////////////////////////////////////////////////////
//
//    Program   : log2ps
//
//    File      : l2p_com.cpp
//
//    Copyright (c) 1998 Ottar Kvindesland
//              This file is a part of the source code of log2ps. It may
//              freely be altered. This copyright notice must however not
//              be removed or changed. The source code may only be
//              distributed further if it is done free of charge, and any
//              changes has been authorized by Ottar Kvindesland
//////////////////////////////////////////////////////////////////////////


#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "struct.h"
#include "o_str.h"
#include "ps.h"
#include "l2p_glob.h"

/*////////////////////////////////////////////////////////////////////////
//
//        band2frq
//
////////////////////////////////////////////////////////////////////////*/

void band2frq (char *b)
{
char f[MAXLINE];

//	f = (char *) malloc ((sizeof (char) * 80));
	strcpy(f, "0");
	switch (atoi(b))
	{
		case 160 : strcpy(f,  "1.8");
		case  80 : strcpy(f,  "3.5");
		case  40 : strcpy(f,    "7");
		case  30 : strcpy(f, "10.1");
		case  20 : strcpy(f,   "14");
		case  17 : strcpy(f,   "18");
		case  15 : strcpy(f,   "21");
		case  12 : strcpy(f,   "26");
		case  10 : strcpy(f,   "28");
		case   2 : strcpy(f,  "144");
	};

	strcpy(b, f);
//	free(f);

};	// band2frq


/*////////////////////////////////////////////////////////////////////////
//
//        readLine
//
////////////////////////////////////////////////////////////////////////*/

void readLine (char *fLine, FILE *inFile)
{
//char *inString;
char inChar;

	fgets(fLine, MAXLINE, inFile);
	if (strlen(fLine) == 0)
		strcpy(fLine, " ");
	cutCR(fLine);
	fileEnd = feof (inFile);

};	// readLine

//////////////////////////////////////////////////////////////////////////
//
//        QSO_hd
//
//////////////////////////////////////////////////////////////////////////

void QSO_hd (char *QSO)
{
char qNo[2];

	crdCntTot++;
	itoa(++crdNo, qNo, 10);
	if (crdNo > cfg.cardsPerSheet.i)
	{
		crdNo = 1;
		itoa(crdNo, qNo, 10);
	};

	strcpy(QSO, "\0");

	strcat(QSO, "posCard");
	if (!((noOfQSLs() == 3) && (crdNo > 1)))
		strcat(QSO, qNo);
	strcat(QSO, " (");

};	/* QSO_hd */

//////////////////////////////////////////////////////////////////////////
//
//        QSO_tail
//
//////////////////////////////////////////////////////////////////////////

void QSO_tail (char *QSO)
{
	strcat(QSO, ") print");
	if (crdNo == cfg.cardsPerSheet.i)
		strcat(QSO, "1");
	strcat(QSO, "card");

};	/* QSO_tail */

//////////////////////////////////////////////////////////////////////////
//
//        makeQSL
//
//////////////////////////////////////////////////////////////////////////

void makeQSL (char *dayStr, char *monStr, char *yrStr,  char *stnStr,
		char *tStr, char *frqStr, char *modStr, char *rptStr)
{
char  outLine[160];
char  year[10];
char  hour[4];
char  min[4];
char  c_hour[4];
char  c_min[4];
char  frq[10];

	getCharStr(frqStr, frq);
	truncAtSpace(stnStr);
	truncAtSpace(rptStr);
	truncAtSpace(modStr);
	truncAtSpace(tStr);

	strcpy(year, cfg.century.s);
	strcat(year, yrStr);

	QSO_hd(outLine);

	strcat(outLine, stnStr);	strcat(outLine, ") (");
	strcat(outLine, rptStr);	strcat(outLine, ") (");
	strcat(outLine, modStr);	strcat(outLine, ") (");
	strcat(outLine, frq);		strcat(outLine, ") (");
	strcat(outLine, tStr);		strcat(outLine, ") (");
	strcat(outLine, dayStr);	strcat(outLine, ") (");
	strcat(outLine, monStr);	strcat(outLine, ") (");
	strcat(outLine, year);

	QSO_tail(outLine);
	PS_outCard (outLine);

	strcpy(cfg.last_yr.s,   year);
	strcpy(cfg.last_mon.s,  monStr);
	strcpy(cfg.last_day.s,  dayStr);
	strcpy(cfg.last_time.s, tStr);

};	// makeQSL

//////////////////////////////////////////////////////////////////////////
//
//        makeEmptyQSL
//
//////////////////////////////////////////////////////////////////////////

void makeEmptyQSL (void)
{
char outLine[160];

	QSO_hd(outLine);

	strcat(outLine, "____________) ( ) ( ) ( ) (  :  ) ( ) ( ) (");
	strcat(outLine, cfg.century.s);

	QSO_tail(outLine);
	PS_outCard (outLine);

};	// makeEmptyQSL

//////////////////////////////////////////////////////////////////////////
//
//        QSL_completeSheet
//
//////////////////////////////////////////////////////////////////////////
void QSL_completeSheet(void)
{
	while (crdNo != cfg.cardsPerSheet.i)
		makeEmptyQSL();

	PS_close();
	printf ("\nFinished, %d cards generated.\n", crdCntTot);

};	/* QSL_completeSheet */

//////////////////////////////////////////////////////////////////////////
//
//        QSL_blanks
//
//////////////////////////////////////////////////////////////////////////
void QSL_blanks(int n)
{
int m = 0;

	PS_print_header ();
	while (m++ < n)
		makeEmptyQSL();
	QSL_completeSheet();

};	/* QSL_completeSheet */

//////////////////////////////////////////////////////////////////////////
//
//        printCard
//
//////////////////////////////////////////////////////////////////////////

void printCard (char *l)
{

char hr[30]   = " ";
char min[30]  = " ";
char time[30] = " ";
char day[30]  = " ";
char mon[30]  = " ";
char yr[30]   = " ";
char wrk[30]  = " ";
char frq[30]  = " ";
char mod[30]  = " ";
char rpt[30]  = " ";
int n;

	getChunk (hr,  l, c_qp.hr);
	getChunk (min, l, c_qp.min);
	getChunk (day, l, c_qp.day);
	getChunk (mon, l, c_qp.mon);
	getChunk (yr,  l, c_qp.yr);
	getChunk (wrk, l, c_qp.wrk);
	getChunk (frq, l, c_qp.frq);
	getChunk (mod, l, c_qp.mod);
	getChunk (rpt, l, c_qp.rpt);

	strcpy(time, hr);
	strcat(time, ":");
	strcat(time, min);

	if (cfg.fixmod.b)
		strcpy(mod, cfg.fixmod.s);
	if (cfg.fixrpt.b)
		strcpy(rpt, cfg.fixrpt.s);

	makeQSL(day, mon, yr, wrk, time, frq, mod, rpt);

};	/* printCard */

//////////////////////////////////////////////////////////////////////////
//
//	lprintCard
//
//	Prints card info to screen, used during testprint
//////////////////////////////////////////////////////////////////////////

void lprintCard (char *l)
{

char hr[30]   = " ";
char min[30]  = " ";
char day[30]  = " ";
char mon[30]  = " ";
char yr[30]   = " ";
char wrk[30]  = " ";
char frq[30]  = " ";
char mod[30]  = " ";
char rpt[30]  = " ";
char ol[300]   = " ";
char i[2]     = "\0";
int n, m;

	getChunk (hr,  l, c_qp.hr);
	getChunk (min, l, c_qp.min);
	getChunk (day, l, c_qp.day);
	getChunk (mon, l, c_qp.mon);
	getChunk (yr,  l, c_qp.yr);
	getChunk (wrk, l, c_qp.wrk);
	getChunk (frq, l, c_qp.frq);
//	if (strcmp(cfg.logProg.s, "CT") == 0)
//		ct_showMod (mod);
//	else
		getChunk (mod, l, c_qp.mod);
	getChunk (rpt, l, c_qp.rpt);

	strcpy(ol, "Day: ");
	strcat(ol, day);
	strcat(ol, ", Mon: ");
	strcat(ol, mon);
	strcat(ol, ", Yr: ");
	strcat(ol, yr);
	strcat(ol, ", Wrk: ");
	strcat(ol, wrk);
	strcat(ol, ", Hr: ");
	strcat(ol, hr);
	strcat(ol, ", Min: ");
	strcat(ol, min);
	strcat(ol, ", Frq: ");
	strcat(ol, frq);
	strcat(ol, ", Mod: ");
	strcat(ol, mod);
	strcat(ol, ", Rpt: ");
	strcat(ol, rpt);

	for (n=0; n<8; n++)
		for (m=0; m<10; m++)
		{	itoa(n, i, 10);
			printf(i);
		};
	for (n=0; n<8; n++)
		for (m=0; m<10; m++)
		{	itoa(m, i, 10);
			printf(i);
		};

	printf("%s\n", l);
	printf("%s\n\n", ol);

};	/* lprintCard */
