//////////////////////////////////////////////////////////////////////////
//
//    Program   : log2ps
//
//    File      : o_str.cpp
//
//    Copyright (c) 1998 Ottar Kvindesland
//              This file is a part of the source code of log2ps. It may
//              freely be altered. This copyright notice must however not
//              be removed or changed. The source code may only be
//              distributed further if it is done free of charge, and any
//              changes has been authorized by Ottar Kvindesland
//////////////////////////////////////////////////////////////////////////

#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "struct.h"
//#include "qsltypes.h"

//////////////////////////////////////////////////////////////////////////
//
//        charsinstr
//
//	  Counts occurences of a given character in string
//
//////////////////////////////////////////////////////////////////////////

int charsinstr (char *in, char c)
{
int n = 0;

	while (*in != '\0')
		if (*in++ == c)
			n++;

	return (n);


};	/* charsinstr */

//////////////////////////////////////////////////////////////////////////
//
//        hdtostr
//
//	  Copy instring to outstring. From first character to first
//	  occurence of given character in instring.
//
//////////////////////////////////////////////////////////////////////////

void hdtostr (char *out, char *in, char c)
{
	while ((*in != '\0') && (*in != c))
		*out++ = *in++;
	++*out = '\0';
};	/* hdtostr */

//////////////////////////////////////////////////////////////////////////
//
//        tailtostr
//
//	  Copy instring to outstring from after first occurence of given
//	  character to end of outstring.
//
//////////////////////////////////////////////////////////////////////////

void tailtostr (char *out, char *in, char c)
{

	while ((*in != '\0') && (*in != c))
		*in++;
	if (*in == c)
		*in++;
	while ((*out++ = *in++) != c)
		;
};	/* tailtostr */

//////////////////////////////////////////////////////////////////////////
//
//	isint
//
//	Return true when parameter represents positive or negative
//	integer.
//
//////////////////////////////////////////////////////////////////////////

bool isint (char *instr)
{
bool r;
int n;
char c;
char *in;

	in = instr;
	if (*in == '-')
		in++;

	n = strlen(in);

	while (--n >= 0)
	{
		c = in[n, n];
		if (!(isdigit(c)))
			return(false);
	};
	return(true);

};	/* isint */

//////////////////////////////////////////////////////////////////////////
//
//	isletter
//
//	Return true when parameter represents a letter A..Z,,,,
//	a..z,,,
//
//////////////////////////////////////////////////////////////////////////

bool isletter (char in)
{
bool r = false;

	if ((isalpha(in)) ||
	    (in == '')   ||
	    (in == '')   ||
	    (in == '')   ||
	    (in == '')   ||
	    (in == '')   ||
	    (in == '') )
		r = true;
	return(r);

};	/* isletter */

//////////////////////////////////////////////////////////////////////////
//
//        numIntStr
//
//	  Return number of integers in string
//
//////////////////////////////////////////////////////////////////////////

int numIntStr (char *instr)
{
int i;
int n=0;

	for (i = 0; i < strlen(instr); i++)
	{       if (isdigit(instr[i]))
			n++;
	};

	return (n);

};	/* numIntStr */

//////////////////////////////////////////////////////////////////////////
//
//        numCharStr
//
//	  Return number of characters A..Z,,,,a..z,,, in string
//
//////////////////////////////////////////////////////////////////////////

int numCharStr (char *instr)
{
int i;
int n=0;

	for (i = 0; i < strlen(instr); i++)
	{       if (isletter(instr[i]))
			n++;
	};

	return (n);

};	/* numCharStr */

//////////////////////////////////////////////////////////////////////////
//
//        numDotStr
//
//	  Return number of desimal points ',','.' in string
//
//////////////////////////////////////////////////////////////////////////

int numDotStr (char *instr)
{
int i;
int n=0;

	for (i = 0; i < strlen(instr); i++)
	{       if ((instr[i] == ',') ||
		    (instr[i] == '.'))
			n++;
	};

	return (n);

};	/* numDotStr */

//////////////////////////////////////////////////////////////////////////
//
//        numHashStr
//
//	  Return number of hashes '/' in string
//
//////////////////////////////////////////////////////////////////////////

int numHashStr (char *instr)
{
int i;
int n=0;

	for (i = 0; i < strlen(instr); i++)
	{       if ((instr[i] == ',') ||
		    (instr[i] == '.'))
			n++;
	};

	return (n);

};	/* numHashStr */

//////////////////////////////////////////////////////////////////////////
//
//        inString
//
//	  Returns true when head of source string conforms to test string.
//
//////////////////////////////////////////////////////////////////////////

byte inString (char *srcStr, char *tstStr)
{
char *tstPtr, *srcPtr;

	tstPtr = tstStr;
	srcPtr = srcStr;
	while (*tstPtr != 0)
	{
		if (*tstPtr != *srcPtr)
			return false;
		tstPtr ++;
		srcPtr ++;
	};
	return true;

};	// inString

/*////////////////////////////////////////////////////////////////////////
//
//        cutCR
//
//	  replaces carriage return (line feed) with NULL
//
////////////////////////////////////////////////////////////////////////*/

void cutCR (char *l)
{
char *pl;

	pl = l;
	while (*pl != '\0')
	{	if (*pl == '\n')
			*pl = '\0';
		else
			pl++;
	};

};	// cutCR

/*////////////////////////////////////////////////////////////////////////
//
//        truncAtSpace
//
//	  Places a NULL at first space
//
////////////////////////////////////////////////////////////////////////*/

void truncAtSpace (char *l)
{
char *pl;

	pl = l;
	while (*pl != '\0')
	{	if (*pl == ' ')
			*pl = '\0';
		else
			pl++;
	};

};	// truncAtSpace

/*////////////////////////////////////////////////////////////////////////
//
//        getCharStr
//
//	  copies first word in string.
//
////////////////////////////////////////////////////////////////////////*/

void getCharStr (char *in, char *out)
{
	while (*in == ' ')
		*in++;
	truncAtSpace(in);
	strcpy(out, in);

};	// getCharStr

/*////////////////////////////////////////////////////////////////////////
//
//        subStr
//
//	  Copies substr of input string to output string.
//
////////////////////////////////////////////////////////////////////////*/

void subStr (char *in, char *out, int h, int t)
{
int  n = 0;
char *pin;
char *pout;

	pin = in;
	pout = out;

	while (n++ < h)
		*pin++;
	n--;
	while (n++ <= t)
		*pout++ = *pin++;
	strcat(out, "\0");

};	// getCharStr


//////////////////////////////////////////////////////////////////////////
//
//        replus
//
//	  replaces underscores with spaces
//
//////////////////////////////////////////////////////////////////////////

char * replus (char *s)
{
char *ls;

	ls = s;

	while (*s != '\0')
	{
		if (*s == '_')
			*s = ' ';
		*s++;
	};

	return (ls);

};	/* replus */

//////////////////////////////////////////////////////////////////////////
//
//        is_0or1
//
//////////////////////////////////////////////////////////////////////////

bool is_0or1 (char *s)
{
	return ((strlen(s) == 1) && ((*s == '0') || (*s == '1')) );

};	/* is_0or1 */

//////////////////////////////////////////////////////////////////////////
//
//        maxLenOK
//
//////////////////////////////////////////////////////////////////////////

bool maxLenOK (char *s, int l)
{
	return (strlen(s) < l);

};	/* maxLenOK */

//////////////////////////////////////////////////////////////////////////
//
//        isDOSfileName
//
//////////////////////////////////////////////////////////////////////////

bool isDOSfileName (char *s)
{
char hd[80];
char tl[80];

	switch (charsinstr(s, '.'))
	{
		case 0 : return (strlen(s) < 9);
		case 1 : hdtostr (hd, s, '.');
			 tailtostr (tl, s, '.');
			 return ((strlen(hd) < 9) && (strlen(tl) < 4));
		default: return(false);
	};

};	/* isDOSfileName */

//////////////////////////////////////////////////////////////////////////
//
//        getGroupPos
//
//////////////////////////////////////////////////////////////////////////

int getGroupPos (int p, char *l)
{
char *lp;
int g_count, l_count, res;

	lp  = l;
	res = 0;

	for (g_count = 0; g_count < p; g_count++)
	{
		while ((*lp == ' ') && (*lp != '\n') && (*lp != '\0'))
		{	lp++;
			res++;
		};

		while ((*lp != ' ') && (*lp != '\n') &&
		       (p != g_count + 1) && (*lp != '\0'))
		{	lp++;
			res++;
		};

	};
	return (res);

};	/* getGroupPos */

//////////////////////////////////////////////////////////////////////////
//
//        getGroup
//
//////////////////////////////////////////////////////////////////////////

void getGroup (char *in, char *out, int p)
{
int n;
int m = 0;
char *inp;

	inp = in;
	n = getGroupPos(p, in);
	for (p=0; p<n; p++)
		inp++;

	while ((in[n] != ' ') && (in[n] != '\0'))
		out[m++] = in[n++];
	out[n] = '\0';

};	/* getGroup */

//////////////////////////////////////////////////////////////////////////
//
//        getChunk
//
//        Parameters:
//          c : chunk
//          l : line
//          p : position
//
//////////////////////////////////////////////////////////////////////////

bool getChunk (char *c, char *l, tlpos q)
{
char *lp, *s, *sp;
int  hd, tl, n;

	lp = l;

	s  = (char *) malloc ((sizeof (char) * MAXLINE));
	sp = s;

	switch (q.tl_t)
	{	case 'A' : tl = q.tl; break;
		case 'E' : tl = getGroupPos(q.tl, l);
			   for (n = 0; n < tl; n++)
				lp++;
			   while ((*lp != ' ') && (*lp != '\n') &&
				  (*lp != '\0'))
			   {
				lp++;
				tl++;
				if ((*lp == ' ') || (*lp == '\n') ||
				    (*lp == '\0'))
					tl--;
			   };

	};

	switch (q.hd_t)
	{	case 'A' : hd = q.hd; break;
		case 'G' : hd = getGroupPos(q.hd, l); break;
	};

	lp = l;

	for (n = 0; n < hd; n++)
		lp++;
	for (n = hd; n <= tl; n++)
		*sp++ = *lp++;
	*sp = '\0';

	sp = s;
	while ((*sp == ' ') && (*sp != '\0'))
		sp++;

	strcpy(c, sp);

	free(s);
	return (hd + tl >= 0);

};	/*getChunk */


//////////////////////////////////////////////////////////////////////////
//
//        formatIntStr
//
//        Formating integer to ascii with leading zeroes
//
//////////////////////////////////////////////////////////////////////////

void formatIntStr (int n, int w, char *s)
{
char num[80];
int  l;
//	sp = s;
	itoa(n, num, 10);
	l = strlen(num);
	while (w > l++)
	{	strcat(s, "0\0");
	};
	strcat(s, num);
	strcat(s, "\0");
};	/*formatIntStr */


