//////////////////////////////////////////////////////////////////////////
//
//    Program   : log2ps
//
//    File      : log-eqf.cpp
//
//    Copyright (c) 1998 Ottar Kvindesland
//              This file is a part of the source code of log2ps. It may
//              freely be altered. This copyright notice must however not
//              be removed or changed. The source code may only be
//              distributed further if it is done free of charge, and any
//              changes has been authorized by Ottar Kvindesland
//////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include "struct.h"
#include "o_files.h"
#include "l2p_glob.h"
#include "l2p_com.h"
#include "o_str.h"

char logLine[MAXLINE];
FILE *genLogFile;


//////////////////////////////////////////////////////////////////////////
//
//	maxPos
//
//	Returns max used position on log line
//////////////////////////////////////////////////////////////////////////

int maxPos(void)
{
int lastPos;

	lastPos = c_qp.wrk.tl;
	if (c_qp.wrk.tl < c_qp.yr.tl)
		lastPos = c_qp.yr.tl;
	if (c_qp.yr.tl < c_qp.mon.tl)
		lastPos = c_qp.mon.tl;
	if (c_qp.mon.tl < c_qp.day.tl)
		lastPos = c_qp.day.tl;
	if (c_qp.day.tl < c_qp.hr.tl)
		lastPos = c_qp.hr.tl;
	if (c_qp.hr.tl < c_qp.min.tl)
		lastPos = c_qp.min.tl;
	if (c_qp.min.tl < c_qp.frq.tl)
		lastPos = c_qp.frq.tl;
	if (c_qp.frq.tl < c_qp.mod.tl)
		lastPos = c_qp.mod.tl;
	if (c_qp.mod.tl < c_qp.rpt.tl)
		lastPos = c_qp.rpt.tl;

	return (lastPos);
};	/* maxPos */

//////////////////////////////////////////////////////////////////////////
//
//	is____Str
//
//	Returns true if element in string is legal QSO element
//////////////////////////////////////////////////////////////////////////

bool isCallStr(char *line, tlpos q)
{
bool res = false;
int n;
int l;
int o;

char s[80] = "\0";

	subStr (line, s, q.hd, q.tl);
	if (q.hd_t == 'A')
		truncAtSpace(s);
	n = numIntStr(s);
	l = numCharStr(s);
	o = numHashStr(s);

	if ((n > 0) &&
	    (l > 1) &&
	    (strlen(s) == (n + l + o)) )
	   res = true;

	return (res);

};	/* isCallStr */

bool isIntStr(char *line, tlpos q)
{
bool res = false;
int n;

char s[80] = "\0";

	subStr (line, s, q.hd, q.tl);
	truncAtSpace(s);
	n = numIntStr(s);

	if (n == strlen(s))
	   res = true;

	return (res);

};	/* isIntStr */

bool isDayStr(char *line, tlpos q)
{
bool res = false;
int n;

char s[80] = "\0";

	subStr (line, s, q.hd, q.tl);
	truncAtSpace(s);
	if (isIntStr(line, q))
	{	n = atoi(s);
		if (n < 32)
			res = true;
	};

	return (res);

};	/* isDayStr */

bool isHrStr(char *line, tlpos q)
{
bool res = false;
int n;

char s[80] = "\0";

	subStr (line, s, q.hd, q.tl);

	truncAtSpace(s);
	if (isIntStr(line, q))
	{	n = atoi(s);
		if (n < 25)
			res = true;
	};

	return (res);

};	/* isHrStr */

bool isMinStr(char *line, tlpos q)
{
bool res = false;
int n;

char s[80] = "\0";

	subStr (line, s, q.hd, q.tl);

	truncAtSpace(s);
	if (isIntStr(line, q))
	{	n = atoi(s);
		if (n < 61)
			res = true;
	};

	return (res);

};	/* isMinStr */

bool isFrqStr(char *line, tlpos q)
{
bool res = false;
int n;
int l;
int o;

char s[80] = "\0";

	subStr (line, s, q.hd, q.tl);

	truncAtSpace(s);
	n = numIntStr(s);
	l = numCharStr(s);
	o = numDotStr(s);

	if ((n >  0) &&
	    (l == 0) &&
	    (strlen(s) == (n + l + o))
	   )
	   res = true;

	return (res);

};	/* isFrqStr */

//////////////////////////////////////////////////////////////////////////
//
//        isQSO
//
//////////////////////////////////////////////////////////////////////////

byte isQSO (char *eqfLogLine, FILE *eqfLogFile)
{
char c;
bool ok;
bool testing;
bool res;
int n;

	readLine(eqfLogLine, eqfLogFile);
	n       = 0;
	testing = true;
	res     = true;
	ok      = true;

	if (strlen(eqfLogLine) < 50)
	{
		res     = false;
		testing = false;
	};

	while (testing)
	{
		c = eqfLogLine[n, n];
		if (c == '\n')
			testing = false;
		else
		{	switch (n)

			{
				case 0  : ok = (isdigit(c)); break;
				case 1  : ok = (isdigit(c)); break;
				case 6  : ok = (isdigit(c)); break;
				case 7  : ok = (isdigit(c)); break;
				case 8  : ok = (c == ' '); break;
				case 9  : ok = (isdigit(c)); break;
				case 10 : ok = (isdigit(c)); break;
				case 12 : ok = (isdigit(c)); break;
				case 13 : ok = (isdigit(c)); break;
				case 14 : ok = (c == ' '); break;
				case 15 : testing = false;
			};
			if (ok == false)
			{
				testing = false;
				res  = false;
			};
		};
		n++;
	};
	return(res);


};	// el_isQSO

//////////////////////////////////////////////////////////////////////////
//
//        qp_testlog
//
//////////////////////////////////////////////////////////////////////////

void qp_testlog (void)
{
int lastPos;

	fileEnd = false;
	genLogFile = fopen (cfg.logFileName.s, "r");

	printf("\nPrinting elements from log. Only lines with data as\n");
	printf("defined -QP... considered\n\n");

	if (file_exist (cfg.logFileName.s))
		fseek(genLogFile, 0, SEEK_SET);
	else
		fileEnd = true;

	while (!fileEnd)
	{	readLine(logLine, genLogFile);
		if (strlen(logLine) >= maxPos())
			lprintCard(logLine);
	};

	fclose (genLogFile);

};	// qp_testlog

//////////////////////////////////////////////////////////////////////////
//
//        other_readLog
//
//////////////////////////////////////////////////////////////////////////

byte other_readLog (void)
{
int lastPos;
byte legCrd;

	genLogFile = fopen (cfg.logFileName.s, "r");
	fileEnd = false;

	lastPos = maxPos();

	if (file_exist (cfg.logFileName.s))
		fseek(genLogFile, 0, SEEK_SET);
	else
		fileEnd = true;

	while (!fileEnd)
	{	readLine(logLine, genLogFile);
		legCrd = (strlen(logLine) >= lastPos);
		if (legCrd)
		{
		legCrd = (legCrd && isCallStr(logLine, c_qp.wrk));
		legCrd = (legCrd && isIntStr(logLine, c_qp.yr));
		legCrd = (legCrd && isDayStr(logLine, c_qp.day));
		legCrd = (legCrd && isHrStr(logLine, c_qp.hr));
		legCrd = (legCrd && isMinStr(logLine, c_qp.min));
//		legCrd = (legCrd && isFrqStr(logLine, c_qp.frq));
		if (legCrd)
			printCard (logLine);
		};
	};


	fclose (genLogFile);
	return(true);

};	// other_readLog

